#include "utils.h"

void setLed(unsigned char pin, unsigned char ledStates[2], unsigned char state) {
    int index = pin / 8;
    pin %= 8;
    pin = 7 - pin;

    if (index == 0 && pin != 7) {
        pin = 7 - pin - 1;
    }

    if (state) {
        bitSet(ledStates[index], pin);
    } else {
        bitClear(ledStates[index], pin);
    }

    updateShiftRegister(ledStates);
}

void updateShiftRegister(unsigned char val[2])
{
    digitalWrite(SR_RCLK, LOW);
    shiftOut(SR_SER, SR_SRCLK, LSBFIRST, val[1]);
    shiftOut(SR_SER, SR_SRCLK, LSBFIRST, val[0]);
    digitalWrite(SR_RCLK, HIGH);
}

bool muxRead(int pin) {
    digitalWrite(MUX_S0, bitRead(pin, 0));
    digitalWrite(MUX_S1, bitRead(pin, 1));
    digitalWrite(MUX_S2, bitRead(pin, 2));
    digitalWrite(MUX_S3, bitRead(pin, 3));

    return digitalRead(MUX_SIG);
}

void controlChange(unsigned char channel, unsigned char control, unsigned char value) {
    midiEventPacket_t event = {0x0B, 0xB0 | channel, control, value};
    MidiUSB.sendMIDI(event);
}

void noteOn(unsigned char channel, unsigned char pitch, unsigned char velocity) {
    midiEventPacket_t noteOn = {0x09, 0x90 | channel, pitch, velocity};

    MidiUSB.sendMIDI(noteOn);
}

void noteOff(unsigned char channel, unsigned char pitch, unsigned char velocity) {
    midiEventPacket_t noteOff = {0x08, 0x80 | channel, pitch, velocity};

    MidiUSB.sendMIDI(noteOff);
}

int getNote(int presetNumber, int index) {
    if (index == 1) {
        index = 2;
    } else if (index == 2) {
        index = 4;
    } else if (index == 3) {
        index = 5;
    } else if (index > 3) {
        index += 3;
    }
    return notePitch[87 - 3 - presetNumber * 12 - index];
}

void getNoteName(int presetNumber, int index, char *name) {
    const char *notes[12] = {"C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B"};

    if (index == 1) {
        index = 2;
    } else if (index == 2) {
        index = 4;
    } else if (index == 3) {
        index = 5;
    } else if (index > 3) {
        index += 3;
    }

    int octave = (presetNumber * 12 + index) / 12 + 1;
    sprintf(name, "%s%d", notes[index], octave);
}

int getControl(int presetNumber, int index) {
    return index + presetNumber * NUM_BUTTONS_MIDI;
}

void turnOffAll(int channel, unsigned char ledStates[2], int presetNumber) {
    for (int button = 0; button < NUM_BUTTONS; button++) {
        setLed(button, ledStates, 1);
        noteOff(channel, getNote(presetNumber, button), 0);
    }
}
